<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class RevisaoQuantidade extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $pratoService;
    protected $newCode;
    protected $enviroment;

    protected $parameterBagColumns = array();
    protected $dataSourceManager;

    public function __construct(
        $crudService, $parameterBag, $dataSourceManager,
        \Teknisa\PLA\Modules\Prato\Services\PratoService $pratoService,
        \Teknisa\GER\Utilities\Environment $enviroment,
        \Teknisa\GER\Services\NewCode $newCode
    ) {
        parent::__construct($crudService, $parameterBag);
        $this->pratoService = $pratoService;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->enviroment = $enviroment;
    }

    public function revisarQtdPorPessoaAut(
        \Zeedhi\Framework\DTO\Request\Row $request,
        \Zeedhi\Framework\DTO\Response $response
    ) {
        try {
            $this->pratoService->iniciaTransacao();
            $codLista = $this->newCode->getCodeByTableName('LISTA', $this->enviroment->getNrOrgTrab());

            $row = $request->getRow();

            $tipoReceita = $row['TIPORECEITA'];
            $fator = $row['FATOR'];
            $qtd = $row['QTD'];
            $rows = $row['ROWS'];

            if($fator == 'P') {
                $qtd =  1 + ($qtd/100);    
            }
            
            if (isset($row["CDSERVFILI"])) {
                $servico = $row['CDSERVFILI'];
                $filial = $row['CDFILIAL'];
            } else if (array_key_exists("CDFILIAL", $row)) {
                $servico = 'X';
                $filial = $row['CDFILIAL'];
            } else {
                $servico = 'X';
                $filial = 'X';
            }

            $data = $this->enviroment->getCurrentTime()->format('d/m/Y');

            foreach ($rows as $pratos) {
                $dados = array('NRLISTA' => $codLista, 'CDLISTA' => $pratos['CDFILAUXPRAT'].$pratos['CDPRATO'], '__is_new' => true, 'DTULTATULIST' => $data);
                $requestLista = new \Zeedhi\Framework\DTO\Request\Row($dados, '', '', '');
                $dadosLista = $this->crudService->getRowsToSave($requestLista, "lista");
                $this->crudService->save("lista", $dadosLista);
            }

            $conn = $this->pratoService->getConnection();
            $revisao = new StoredProcedure($conn, 'REVISA_QTDE');

            $revisao->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $codLista, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_IDTIPORECE', Param::PARAM_INPUT, $tipoReceita, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_IDOPCAO', Param::PARAM_INPUT, $fator, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_VRVALOR', Param::PARAM_INPUT, $qtd, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $servico, Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, 'X', Param::PARAM_TYPE_STR));
            $revisao->addParam(new Param('P_CDTIPOCARD', Param::PARAM_INPUT, 'X', Param::PARAM_TYPE_STR));

            $revisao->execute();

            $response->addDataset(new DataSet_Source('RETORNO', array(true)));

            $this->pratoService->commit();
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        } finally {
            $condition = array(
                array(
                    "columnName" => "NRLISTA",
                    "operator" => "=",
                    "value" => $codLista
                )
            );

            $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('lista', $condition);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $this->dataSourceManager->delete($dataSet);
        }
    }
}