<?php
namespace Teknisa\PLA\Modules\Requisicao\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\DataSet;

class Pla01600RequisicaoPorSetor extends \Zeedhi\Framework\Controller\Report {

    protected $qr2Strategy;
    protected $novoCodigo;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $dataSourceManager;
    protected $util;
    protected $libUtilities;
    
    public function __construct($reportService, $novoCodigo, $connection, $environment, $qr2Strategy, $birtStrategy, $crudService, $dataSourceManager, $util,$libUtilities) {
        $this->reportService = $reportService;
        $this->novoCodigo    = $novoCodigo;
        $this->connection    = $connection;
        $this->environment   = $environment;
        $this->qr2Strategy   = $qr2Strategy;
        $this->birtStrategy  = $birtStrategy;
        $this->crudService   = $crudService;
        $this->dataSourceManager   = $dataSourceManager;
        $this->util = $util;
        $this->libUtilities  = $libUtilities;
    }

    protected $reportMapping = array (
        'PLA01600' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01601' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01602' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'CDPICTPRAT'     => 'P_CDPICTPRAT',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01603' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01604' =>
            array (
                'strategy' => 'QR2',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA'
                    )
            ),
            'PLA01605' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPRAT'     => 'P_CDPICTPRAT',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01606' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01607' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            ),
            'PLA01608' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRCONSPLAN'     => 'P_NRCONSPLAN',
                        'CDFILISERV'     => 'L_CDSERVFILI',
                        'CDGRUPSERV'     => 'P_CDGRUPSERV',
                        'TCDGRUPSERV'    => 'P_TCDGRUPSERV',
                        'CDGRUPPROD'     => 'P_CDGRUPPROD',
                        'TCDGRUPPROD'    => 'P_TCDGRUPPROD',
                        'CDARVPRODINI'   => 'P_PRODINI',
                        'CDARVPRODFIN'   => 'P_PRODFIN',
                        'NMPRODINI'      => 'P_NMPRODINI',
                        'NMPRODFIN'      => 'P_NMPRODFIN',
                        'TIPONIVEL'      => 'P_TIPONIVEL',
                        'DTINICIAL'      => 'P_DTINICIAL',
                        'DTFINAL'        => 'P_DTFINAL',
                        'TAMNIVELPROD'   => 'P_TAMNIVELPROD',
                        'TAMNIV'         => 'P_TAMNIV',
                        'ORDENA'         => 'P_ORDENA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'NRLINHABRANCA'  => 'P_NRLINHABRANCA',
                        'CDOPERADOR'     => 'P_CDOPERADOR',
                        'NMOPERADOR'     => 'P_NMOPERADOR',
                        'CDSETOR'        => 'L_CDSETOR',
                        'IDCARD'         => 'P_IDCARD',
                        'IDFIXO'         => 'P_IDFIXO',
                        'IDPERC'         => 'P_IDPERC',
                        'CDPONTOENTREGA' => 'L_CDPONTOENTREGA',
                        'CDPICTPROD'     => 'P_CDPICTPROD',
                        'NRORG'          => 'P_NRORG'
                    )
            )
       
        
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        
            $row = $request->getRow();
            if($row["__report_name"] != 'PLA01604'){
                $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
            }else{
                $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
            }
            if($row['TCDFILIAL'] == 'T')$row['CDFILIAL'] = "T";
            else $row['CDFILIAL'] = $this->concatParams($row,"CDFILIAL");
            
          
            
            
            if($row['TCDGRUPSERV'] == 'T')$row['CDGRUPSERV'] = "T";
            else $row['CDGRUPSERV'] = $this->concatParams($row,"NMGRUPSERV");
            
            if($row['TCDSERVICO'] == 'T'){
                $result =  self::setDataSourceServico($request,  $response, true);
            
                $cdServFili = "";
                foreach($result as $key => &$filial){
                    $cdServFili.=$filial['CDFILISERV']."_";
                }
                $row['CDFILISERV'] = $cdServFili;
            }else{
                $row['CDFILISERV'] = $this->concatParams($row,"CDFILISERV");  
            }
            //Serviços a ignorar conforme rotina de validação do cardápio
            if(isset($row['SERVICOS_A_IGNORAR'])){
                foreach($row['SERVICOS_A_IGNORAR'] as $ignorado){
                    $search = $ignorado['CDFILIAL'].$ignorado['CDSERVFILI']."_";
                    $row['CDFILISERV'] = str_replace($search, "", $row['CDFILISERV']);
                }
            }
            $nrlista = $this->saveLista($row['CDFILISERV']);

            if($row["__report_name"] != 'PLA01604'){
                $row['CDFILISERV'] =  $this->saveLista($row['CDFILISERV']);
                
                if($row['TCDSETOR'] == 'T'){
                    $result =  self::setDataSourceSetor($request,  $response, true);
                    
                    $cdFiliSetor = "";
                    foreach($result as $key => &$filial){
                        $cdFiliSetor.=$filial['CDFILISETOR']."_";
                    }
                    $row['CDSETOR'] = $cdFiliSetor;
                    $row['CDSETOR'] = $this->saveLista($row['CDSETOR']);
                }else{
                    $row['CDSETOR'] = $this->concatParams($row, "CDFILISETOR");
                    $row['CDSETOR'] = $this->saveLista($row['CDSETOR']);
                }
                if($row['TCDDESTFILI'] == 'T') {
                    $result =  self::setDataSourcePontoEntrega($request,  $response, true);
                    $cdServCliePe = "";
                    foreach($result as $key => &$servcliepe){
                        $cdServCliePe.=$servcliepe['CLIPENT']."_";
                    }
                    $row['CDPONTOENTREGA'] = $cdServCliePe;
                    $row['CDPONTOENTREGA'] = $this->saveLista($row['CDPONTOENTREGA']);
                }
                else {
                    $row['CDPONTOENTREGA'] = $this->concatParams($row,"CLIPENT");
                    $row['CDPONTOENTREGA'] = $this->saveLista($row['CDPONTOENTREGA']);
                }
                
                if($row['TCDGRUPPROD'] == 'T') {
                    $row['CDGRUPPROD'] = "T";
                }
                else {
                    $row['CDGRUPPROD'] = $this->concatParams($row,"CODIGOSUBGRUP");
                    $row['TCDGRUPPROD'] = null;
                }
            }else{
                if($row['TCDSETOR'] == 'T') $row['CDSETOR'] = 'T';
                else $row['CDSETOR'] = $this->concatParams($row,"NMSETORCOZ");
                
                if($row['TCDDESTFILI'] == 'T') $row['CDPONTOENTREGA'] = 'T';
                else {
                    $row['CDPONTOENTREGA'] = $this->concatParams($row,"NMDESTFILI");
                    $row['CDPONTOENTREGA'] = $this->saveLista($row['CDPONTOENTREGA']);
                }
                
                if($row['TCDGRUPPROD'] == 'T') $row['CDGRUPPROD'] = "T";
                else $row['CDGRUPPROD'] = $this->concatParams($row,"NMSUBGRPROD");
            }
            
        
    
             $filiais=$this->retornaFiliais($nrlista);
             $filiais=explode('_',$filiais);
            //coleta de dados
            $this->libUtilities->saveCollectedData( $filiais,'REQUISICAO');
        
            $consplan = $this->verificaParametrizacaoFilial($nrlista);
            $filialLogada =  $this->environment->getCdFilial();
            $row['NRCONSPLAN'] = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$filialLogada ,$this->environment->getNrOrgTrab());
            $row['NRCONSPLAN'][0] = "P";
            $procedureReqAnalitica = new StoredProcedure($this->connection, 'REQUISICAO_ANALITICA_SETOR_PLA');

            $procedureReqAnalitica->addParam(new Param('P_FILIAL',          Param::PARAM_INPUT, "T",                    Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_SERVICOS',        Param::PARAM_INPUT, $nrlista,               Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_DTINICIAL',       Param::PARAM_INPUT, $row['DTINICIAL'],      Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_DTFINAL',         Param::PARAM_INPUT, $row['DTFINAL'],        Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_IDARQUIVO',       Param::PARAM_INPUT, $row['IDARQUIVO'],      Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_IDPRODUTOS',      Param::PARAM_INPUT, $row['IDPRODUTOS'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_IDSEPPRATO',      Param::PARAM_INPUT, $row['IDSEPPRATO'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_PEACRESCIMO',     Param::PARAM_INPUT, $row['PEACRESCIMO'],    Param::PARAM_TYPE_INT));
            $procedureReqAnalitica->addParam(new Param('P_IDARREDOND',      Param::PARAM_INPUT, $row['IDARREDOND'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_NRCONSPLAN',      Param::PARAM_INPUT, $row['NRCONSPLAN'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_CDPRODUTOINI',    Param::PARAM_INPUT, $row['CDARVPRODINI'],      Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_CDPRODUTOFIN',    Param::PARAM_INPUT, $row['CDARVPRODFIN'],      Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT, $row['CDOPERADOR'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_DTHRCALCPR',      Param::PARAM_INPUT, $row['DTHRCALCPR'],     Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_STDTINICIAL',     Param::PARAM_INPUT, $row['STDTINICIAL'],    Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->addParam(new Param('P_STDTFINAL',       Param::PARAM_INPUT, $row['STDTFINAL'],      Param::PARAM_TYPE_STR));
            $procedureReqAnalitica->execute();
            return $nrlista;
            
    }
    
    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $nrlista = $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } finally{
            if(isset($nrlista)) $this->util->deleteLista($nrlista);
        }
    }
    
    public function saveLista($row){
          $listafiliserv = explode("_",$row);
          $nrlista = $this->novoCodigo->getCodeByTableName('LISTA',$this->environment->getNrOrgTrab());
          $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
          foreach ($listafiliserv as $filiserv) {
                $stmt = $this->connection->prepare($sql);
                
                if(!empty($filiserv)){
                    $stmt->execute(
                        array(
                            "NRLISTA" => $nrlista,
                            "CDLISTA" => $filiserv
                        )
                    );     
                }
          }
          return $nrlista;
    }
    
    
    public function montaListaUnidade($row){
        $param = array(
            "CDGRUPFILI" => $row['CDGRUPFILI']
        );
        $query = "SELECT F.CDFILIAL FROM FILIAL F LEFT JOIN FILGRUPFILI FG ON F.CDFILIAL = FG.CDFILIAL 
                  WHERE((FG.CDGRUPFILI = :CDGRUPFILI) OR (:CDGRUPFILI IS NULL)) ORDER BY F.CDFILIAL";
        $result = $this->connection->fetchAll($query, $param);
        
        $lista = "";
        foreach($result as $value){
            $lista .= $value['CDFILIAL']."_";
        }
        return $lista;
    }
    
    public function montaListaGrupserv($row){
        $param = array();
        $query = "SELECT CDGRUPSERV FROM GRUPSERV";
        $result = $this->connection->fetchAll($query, $param);
        
        $lista = "";
        foreach($result as $value){
            $lista .= $value['CDGRUPSERV']."_";
        }
        return $lista;
    }
    
    public function montaListaGrupProd($row){
        $param = array();
        $query = "SELECT CDSUBGRPROD FROM SUBGRUPPROD";
        $result = $this->connection->fetchAll($query, $param);
       
        $lista = "";
        foreach($result as $value){
            $lista .= $value['CDSUBGRPROD']."_";
        }
        return $lista;
    }
    
    public function verificaParametrizacaoFilial($nrlista){
        $param = array(
            "NRLISTA" => $nrlista
        );
        $query = "SELECT DISTINCT IDUTREQUPRAT, CDTPRCONSFIX, CDPRACONSFIX, CDTPRCONSPER,
                 CDPRACONSPER, CDFILIAL, CDPARPLANEJA, IDORDEREQU,
                 NRNIVEREQU, CDTPRETIPADR, IDVISUAQTEST
                 FROM PARAMPLANEJA, LISTA
                 WHERE LISTA.NRLISTA = :NRLISTA
                 AND PARAMPLANEJA.CDFILIAL = SUBSTR(LISTA.CDLISTA, 1, 4)";
        $result = $this->connection->fetchAll($query, $param);
        foreach($result as $value){
           if(empty($value['CDTPRCONSFIX'])) throw new \Exception('Tipo de prato default de consumo fixo deve ser parametrizado');
           if(empty($value['CDPRACONSFIX'])) throw new \Exception('Prato default de consumo fixo deve ser parametrizado');
           if(empty($value['CDTPRCONSPER'])) throw new \Exception('Tipo de prato default de consumo percapita deve ser parametrizado');
           if(empty($value['CDPRACONSPER'])) throw new \Exception('Prato default de consumo percapita deve ser parametrizado');
        }
        
        return $result;
    }
    
    public function retornaServicos(\Zeedhi\Framework\DTO\Request\Filter $request,
    	\Zeedhi\Framework\DTO\Response $response){
    	
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();

        $data = $this->servicoOperador($conditions,$page,$pageSize); 
        if(empty($data)) $data = $this->servico($conditions,$page,$pageSize);
        $response->addDataSet($data);
    }
    
    private function servicoOperador($conditions,$page,$pageSize){
        $dataSet = $this->crudService->getFilteredDataSet("pla01600_servico_operador", $conditions,$page,$pageSize);
        return $dataSet;
    }
    
    private function servico($conditions,$page,$pageSize){
        array_shift($conditions);
        $dataSet = $this->crudService->getFilteredDataSet("pla01600_servico", $conditions,$page,$pageSize);
        return $dataSet;
    }
    
    public function concatParams($row, $param){
        $concat = "";
        foreach($row[$param] as $value){
            $concat .= $value."_";
        }
        return $concat;
    }
    
    public function setDataSourceServico(DTO\Request\Row $request, DTO\Response $response, $flag = false){
        $row = $request->getRow();
        $param = array(
            "CDFILIAL"      => isset($row['CDFILIAL']) == false ? "" : $row['CDFILIAL'],
            "CDOPERADOR"    => isset($row['CDOPERADOR']) == false ? "" : $row['CDOPERADOR'],
            "CDGRUPSERV"    => isset($row['CDGRUPSERV']) == false ? "" : $row['CDGRUPSERV'],
            "CDGRUPFILI"      => isset($row['CDGRUPFILI']) == false ? "" : $row['CDGRUPFILI'],
            "TCDGRUPSERV"   => isset($row['TCDGRUPSERV']) == false ? "" : $row['TCDGRUPSERV']
        );
        $query = "SELECT DISTINCT (SO.CDFILIAL||SO.CDSERVFILI) AS CDFILISERV, SO.CDFILIAL, SO.CDSERVFILI, SE.NMSERVICO FROM SERVOPER SO, SERVICO SE, FILIOPER F, FILGRUPFILI G WHERE ((INSTR(:CDGRUPFILI, G.CDGRUPFILI)>0) OR (:CDGRUPFILI = 'T') OR (:CDGRUPFILI IS NULL)) AND G.CDFILIAL(+) = F.CDFILIAL AND ((INSTR(:CDFILIAL, F.CDFILIAL)>0) OR (:CDFILIAL = 'T')) AND F.CDOPERADOR = :CDOPERADOR AND (SO.CDOPERADOR = F.CDOPERADOR) AND (SO.CDFILIAL = F.CDFILIAL) AND ((INSTR(:CDGRUPSERV, SE.CDGRUPSERV) <> 0) OR (:TCDGRUPSERV = 'T')) AND (SE.CDFILIAL = SO.CDFILIAL) AND (SE.CDSERVFILI = SO.CDSERVFILI) AND (SE.IDTIPOSERV <> 'A') AND (SE.IDSERVATIV <> 'N') AND ((SE.IDTPSERVHOSP IS NULL) OR (SE.IDTPSERVHOSP = '0')) ORDER BY SO.CDFILIAL, SO.CDSERVFILI";
        $resultServOper = $this->connection->fetchAll($query, $param);
        
        $param = array(
            "CDFILIAL"      => isset($row['CDFILIAL']) == false ? "" : $row['CDFILIAL'],
            "CDGRUPSERV"    => isset($row['CDGRUPSERV']) == false ? "" : $row['CDGRUPSERV'],
            "CDOPERADOR"    => isset($row['CDOPERADOR']) == false ? "" : $row['CDOPERADOR'],
            "CDGRUPFILI"      => isset($row['CDGRUPFILI']) == false ? "" : $row['CDGRUPFILI'],
            "TCDGRUPSERV"   => isset($row['TCDGRUPSERV']) == false ? "" : $row['TCDGRUPSERV']);
            
        $query = "SELECT DISTINCT S.CDFILIAL FROM SERVICO       S, FILIAL FI, FILIOPER      F, FILGRUPFILI   G WHERE ( ( INSTR(:CDFILIAL, F.CDFILIAL) <> 0 ) OR ( :CDFILIAL = 'T' ) ) AND ( ( INSTR(:CDGRUPFILI, G.CDGRUPFILI) > 0 ) OR ( :CDGRUPFILI = 'T' ) OR ( :CDGRUPFILI IS NULL ) ) AND G.CDFILIAL(+) = F.CDFILIAL AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :TCDGRUPSERV = 'T' ) OR ( :CDGRUPSERV IS NULL ) ) AND ( F.CDOPERADOR = :CDOPERADOR ) AND ( F.CDFILIAL = S.CDFILIAL ) AND (FI.CDFILIAL = F.CDFILIAL) AND (FI.IDCNTRRESC = 'N') ORDER BY S.CDFILIAL";
        
        $resultAllServ = $this->connection->fetchAll($query, $param);
        
        foreach ($resultServOper as $key => &$servOper) {
            foreach($resultAllServ as $key => &$serv){
                if($servOper['CDFILIAL'] == $serv['CDFILIAL']){
                    unset($resultAllServ[$key]);
                }
            }    
        }
        
        $filialServ = "";
        foreach($resultAllServ as $key => &$serv){
            $filialServ.=$serv['CDFILIAL']."_";
        }

        $param = array(
            "CDFILIAL"      => $filialServ,
            "CDGRUPSERV"    => isset($row['CDGRUPSERV']) == false ? "" : $row['CDGRUPSERV'],
            "TCDGRUPSERV"   => isset($row['TCDGRUPSERV']) == false ? "" : $row['TCDGRUPSERV']
        );
        $query = "SELECT (CDFILIAL||CDSERVFILI) AS CDFILISERV, CDFILIAL, CDSERVFILI, NMSERVICO FROM SERVICO WHERE ((INSTR(:CDFILIAL, CDFILIAL)>0) OR (:CDFILIAL = 'T')) AND ((INSTR(:CDGRUPSERV, CDGRUPSERV) <> 0) OR (:TCDGRUPSERV= 'T')) AND (IDTIPOSERV <> 'A') AND ((IDSERVATIV IS NULL) OR (IDSERVATIV <> 'N')) AND ((IDTPSERVHOSP IS NULL) OR (IDTPSERVHOSP = '0')) ORDER BY CDFILIAL, CDSERVFILI";
        $resultServ = $this->connection->fetchAll($query, $param);
        
        $data = array_merge($resultServOper,$resultServ);
        if($flag){
            return $data;
        }else{
            $response->addDataSet(new DataSet ('pla01600_setDataSourceServico', $data));
        }
    }

    public function setDataSourceSetor(DTO\Request\Row $request, DTO\Response $response, $flag = false){
        $row = $request->getRow();
        $param = array(
            "CDFILIAL"      => isset($row['CDFILIAL']) == false ? "" : $row['CDFILIAL']
        );
        $query = "SELECT ADB.CDFILISETOR FROM ( SELECT ( CDFILIAL || CDSETORCOZ ) AS CDFILISETOR FROM SETORCOZ WHERE ( ( INSTR(:CDFILIAL, CDFILIAL) > 0 ) OR :CDFILIAL = 'T' ) UNION SELECT DISTINCT ( CDFILIAL || '  ' ) AS CDFILISETOR FROM FILIAL WHERE ( ( INSTR(:CDFILIAL, CDFILIAL) > 0 ) OR :CDFILIAL = 'T' ) ) ADB GROUP BY ADB.CDFILISETOR ORDER BY ADB.CDFILISETOR";
        
        $data = $this->connection->fetchAll($query, $param);


        return $data;
    }

    public function setDataSourcePontoEntrega(DTO\Request\Row $request, DTO\Response $response, $flag = false){
        $row = $request->getRow();
        $param = array(
            "CDSERVFILI"      => isset($row['CDSERVFILI']) == false ? "T" : $row['CDSERVFILI']
        );
        $query = "SELECT DESTCLI.CDFILIAL||'%'||DESTCLI.CDSERVFILI||'%'||DESTCLI.CDCLIENTE||'%'||DESTCLI.CDDESTFILI AS CLIPENT FROM DESTFILI DEST INNER JOIN DESTSRVCLI DESTCLI ON DESTCLI.CDFILIAL = DEST.CDFILIAL AND DESTCLI.CDDESTFILI = DEST.CDDESTFILI INNER JOIN CLIENTE CLI ON DESTCLI.CDCLIENTE = CLI.CDCLIENTE INNER JOIN SERVICO SERV ON DESTCLI.CDFILIAL = SERV.CDFILIAL AND DESTCLI.CDSERVFILI = SERV.CDSERVFILI LEFT JOIN FILIAL F ON DESTCLI.CDFILIAL = F.CDFILIAL WHERE (INSTR(:CDSERVFILI, DESTCLI.CDFILIAL||DESTCLI.CDSERVFILI) > 0 OR :CDSERVFILI = 'T') ORDER BY DESTCLI.CDFILIAL, DESTCLI.CDDESTFILI, DESTCLI.CDCLIENTE, DESTCLI.CDDESTFILI";
        
        $data = $this->connection->fetchAll($query, $param);


        return $data;
    }
    public function retornaFiliais($nrlista){
           $sql = "SELECT DISTINCT CDFILIAL  FROM FILIAL F INNER JOIN LISTA L ON L.NRLISTA = :P_NRLISTA AND SUBSTR(L.CDLISTA,1,4) = F.CDFILIAL ORDER BY CDFILIAL";
            $param = array(
              "P_NRLISTA"      => $nrlista
        );
        try {
            $filial = $this->connection->fetchAll($sql,$param);
            return implode('_', array_column($filial, 'CDFILIAL'));
        } catch (\Exception $e) {
            throw $e;
        }
    }
}